package fr.inria.infovis.panel;

import fr.inria.infovis.panel.visualization.GraphControlPanel;
import fr.inria.infovis.panel.visualization.IcicleTreeControlPanel;
import fr.inria.infovis.panel.visualization.MatrixControlPanel;
import fr.inria.infovis.panel.visualization.NodeLinkGraphControlPanel;
import fr.inria.infovis.panel.visualization.NodeLinkTreeControlPanel;
import fr.inria.infovis.panel.visualization.ScatterPlotControlPanel;
import fr.inria.infovis.panel.visualization.TreeControlPanel;
import infovis.Visualization;
import infovis.graph.visualization.GraphVisualization;
import infovis.graph.visualization.MatrixVisualization;
import infovis.graph.visualization.NodeLinkGraphVisualization;
import infovis.table.visualization.ScatterPlotVisualization;
import infovis.tree.visualization.IcicleTreeVisualization;
import infovis.tree.visualization.NodeLinkTreeVisualization;
import infovis.tree.visualization.TreeVisualization;
import infovis.tree.visualization.TreemapVisualization;
import infovis.tree.visualization.treemap.TreemapControlPanel;
import infovis.visualization.DefaultVisualization;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author raf
 *
 * A replacement of the ControlPanelFactory of Infovis
 * 
 * 
 */
public class ControlPanelFactory {
    static ControlPanelFactory sharedInstance;
    
    Map creators = new HashMap();

    /**
     * Constructor for ControlPanelFactory.
     */
    public ControlPanelFactory() {
        createDefaults();
    }
    
    protected void createDefaults() {
        put(DefaultVisualization.class,
            new Creator() {
            public AbstractControlPanel create(Visualization visualization, Class c) {
                if (c == DefaultVisualization.class) {
                    return new DefaultControlPanel(visualization);
                }
                return null;
            }
        });
    }

    public static ControlPanelFactory sharedInstance() {
        if (sharedInstance == null) {
            sharedInstance = new ControlPanelFactory();
            sharedInstance.setDefault(
                    GraphVisualization.class,
                    GraphControlPanel.class);
            sharedInstance.setDefault(
                    MatrixVisualization.class, MatrixControlPanel.class);
            sharedInstance.setDefault(
                    NodeLinkGraphVisualization.class,
                    NodeLinkGraphControlPanel.class);
            sharedInstance.setDefault(
                    ScatterPlotVisualization.class,
                    ScatterPlotControlPanel.class);
            sharedInstance.setDefault(
                    TreeVisualization.class,
                    TreeControlPanel.class);
            sharedInstance.setDefault(
                    IcicleTreeVisualization.class, IcicleTreeControlPanel.class);
            
            sharedInstance.setDefault(
                    NodeLinkTreeVisualization.class,
                    NodeLinkTreeControlPanel.class);
            
            sharedInstance.setDefault(
                    TreemapVisualization.class,
                    TreemapControlPanel.class);
    /*        sharedInstance.setDefault(
                    GraphAsTreemapVisualization.class,
                    GraphAsTreemapControlPanel.class);
                    */
        }
        return sharedInstance;
    }
    
    public static void setSharedInstance(ControlPanelFactory shared) {
        sharedInstance = shared;
    }
    
    /**
     * Creates a Control Panel from a Visualization.
     *
     * @param c The Visualization.
     *
     * @return A Control Panel.
     */
    public AbstractControlPanel createControlPanel(Visualization visualization) {
        AbstractControlPanel ret = null;
        Class visClass = Visualization.class;
        for (Class c = visualization.getClass(); visClass.isAssignableFrom(c); c = c.getSuperclass()) {
            Creator creator = get(c);
            if (creator != null)
                ret = creator.create(visualization, c);
            if (ret != null)
                break;
        }
        return ret;
    }

    /**
     * Adds a default creator for a specific kind of visualization.
     *
     * @param c The creator
     */
    public void put(Class c, Creator creator) {
        assert(Visualization.class.isAssignableFrom(c));
        
        creators.put(c, creator);
    }
    
    public void put(Class visClass, Class cpClass) {
        put(visClass, new DefaultCreator(cpClass));
    }
    
    public void setDefault(Class visClass, Class cpClass) {
        if (get(visClass) == null)
            put(visClass, cpClass);
    }
    
    public Creator get(Class c) {
        return (Creator)creators.get(c);
    }

    public interface Creator {
       AbstractControlPanel create(Visualization visualization, Class c);
    }
    
    public static class DefaultCreator implements Creator {
        Class controlPanelClass;
        
        public DefaultCreator(Class c) {
            assert(DefaultControlPanel.class.isAssignableFrom(c));
            controlPanelClass = c;
        }
        
        public AbstractControlPanel create(Visualization visualization, Class c) {
            Class[] parameterTypes = { c };
            Constructor cons;
            try {
                cons = controlPanelClass.getConstructor(parameterTypes);
            }
            catch (NoSuchMethodException e) {
                return null;
            }
            if (cons != null) {
                Object[] args = { visualization };
                try {
                    return (AbstractControlPanel)cons.newInstance(args);
                }
                catch (InstantiationException e) {
                }
                catch (IllegalAccessException e) {
                }
                catch (InvocationTargetException e) {
                }
            }
            return null;
        }
        
    }
}
