
package fr.inria.infovis.panel;

import infovis.Column;
import infovis.Table;
import infovis.Visualization;
import infovis.column.ColumnFilter;
import infovis.column.FilterColumn;
import infovis.metadata.ValueCategory;
import infovis.panel.DynamicQuery;
import infovis.panel.dqinter.DynamicQuerylFactory;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;


/**
 * Panel managing the Dynamic Query controls on the visualized table.
 *
 * @author Jean-Daniel Fekete
 * @version $Revision: 1.3 $
 */
public class DynamicQueryPanel extends AbstractControlPanel
    implements TableModelListener {
    Table        table;
    HashMap      columnDynamicQuery = new HashMap();
    FilterColumn filterColumn;
    ColumnFilter filter;
    JButton      hideFilteredButton;
    JButton      hideSelectedButton;
    JButton      showAllButton;

    /**
     * Constructor for DynamicQueryPanel.
     *
     * @param table the table.
     */
    public DynamicQueryPanel(Visualization vis, Table table) {
        this(vis, table, FilterColumn.findColumn(table, Table.FILTER_COLUMN));
    }

    /**
     * Creates a new DynamicQueryPanel object.
     *
     * @param table the Table to filter.
     * @param filterColumn the FilterColumn.
     */
    public DynamicQueryPanel(Visualization vis, Table table, FilterColumn filterColumn) {
        this(vis, table, filterColumn, null);
    }

    /**
     * Creates a new DynamicQueryPanel object.
     *
     * @param table the Table to filter.
     * @param filterColumn the FilterColumn.
     * @param filter the ColumnFilter.
     */
    public DynamicQueryPanel(Visualization vis,
                             Table table,
                             FilterColumn filterColumn,
                             ColumnFilter filter) {
        super(vis);
        this.table = table;
        this.filterColumn = filterColumn;
        this.filter = filter;
        Box hbox = new Box(BoxLayout.X_AXIS);
        
        hideFilteredButton = new JButton("Hide Filtered");
        hideFilteredButton.addActionListener(this);
        hbox.add(hideFilteredButton);
        hideSelectedButton = new JButton("Hide Selected");
        hideSelectedButton.addActionListener(this);
        hbox.add(hideSelectedButton);
        showAllButton = new JButton("Show All");
        showAllButton.addActionListener(this);
        hbox.add(showAllButton);
                
        add(hbox);
        addMissingControls();
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == hideFilteredButton) {
            getVisualization().hideFilteredRows();
        }
        else if (e.getSource() == hideSelectedButton) {
            getVisualization().hideSelectedRows();
        }
        else if (e.getSource() == showAllButton) {
            getVisualization().showAllRows();
        }
        else
            super.actionPerformed(e);
    }



    /**
     * Creates a default control for the specified column.
     *
     * @param c the column.
     */
    public void createDefaultControl(Column c) {
        int category = ValueCategory.findValueCategory(c);

        if (category != ValueCategory.TYPE_CATEGORIAL) {
            DynamicQuery dq = DynamicQuerylFactory.createDynamicQuery(c);
            if (dq == null)
                return;
            JComponent rs = dq.getComponent();
            setMaximumSize(rs);
            setTitleBorder(rs, c.getName());
            rs.setEnabled(true);
            add(rs);
            columnDynamicQuery.put(c, dq);
            dq.setFilterColumn(filterColumn);
        }
    }

    /**
     * Removes the control associated with a specified column.
     *
     * @param c the column.
     */
    public void removeControl(Column c) {
        DynamicQuery dq = (DynamicQuery)columnDynamicQuery.remove(c);
        if (dq != null) {
            remove(dq.getComponent());
            dq.setFilterColumn(null);
            dq = null; // should dispose it.
        }
    }
    
    public boolean filter(Column c) {
        if (filter != null)
            return filter.filter(c);
        return false;
    }

    /**
     * Adds Dynamic Query controls for all columns in the table unless one already
     * exists.
     */
    public void addMissingControls() {
        Set knownColumn = new TreeSet(columnDynamicQuery.keySet());
        for (int col = 0; col < table.getColumnCount(); col++) {
            Column c = table.getColumnAt(col);
            if (filter(c))
                continue;
            if (!knownColumn.remove(c)) {
                createDefaultControl(c);
            }
        }
        for (Iterator iter = knownColumn.iterator(); iter.hasNext();) {
            Column c = (Column)iter.next();
            removeControl(c);
        }
    }

    /**
     * @see javax.swing.event.TableModelListener#tableChanged(TableModelEvent)
     */
    public void tableChanged(TableModelEvent e) {
        addMissingControls();
    }

    // Filter management
    public int columnOffset(Column column) {
        int offset = table.indexOf(column);
        if (offset < 0) {
            throw new ArrayIndexOutOfBoundsException("column is not in table");
        }
        return offset;
    }

    /**
     * Returns the filterColumn.
     * @return FilterColumn
     */
    public FilterColumn getFilterColumn() {
        return filterColumn;
    }

    /**
     * Sets the filterColumn.
     * @param filterColumn The filterColumn to set
     */
    public void setFilterColumn(FilterColumn filterColumn) {
        this.filterColumn = filterColumn;
    }

    /**
     * Returns the dynamic query associated with a specified column.
     *
     * @param c the column.
     *
     * @return the dynamic query associated with a specified column or null if
     *  no dynamic query exist for this column.
     */
    public DynamicQuery getColumnDynamicQuery(Column c) {
        return (DynamicQuery)columnDynamicQuery.get(c);
    }
    /**
     * Returns the filter.
     * @return ColumnFilter
     */
    public ColumnFilter getFilter() {
        return filter;
    }

    /**
     * Sets the filter.
     * @param filter The filter to set
     */
    public void setFilter(ColumnFilter filter) {
        this.filter = filter;
        addMissingControls();
    }

}
