/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 16 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.event;

import java.lang.reflect.Array;



/**
 * @author raf
 *
 * @
 */
public abstract class SlowerEventListenerManagerImpl implements  EventListenerManager {
    
    protected transient Object[] listenerList = NULL_ARRAY;
    
    /**
     * Passes back the event listener list as an array
     * of ListenerType-listener pairs.  Note that for 
     * performance reasons, this implementation passes back 
     * the actual data structure in which the listener data
     * is stored internally!  
     * This method is guaranteed to pass back a non-null
     * array, so that no null-checking is required in 
     * fire methods.  A zero-length array of Object should
     * be returned if there are currently no listeners.
     * 
     * WARNING!!! Absolutely NO modification of
     * the data contained in this array should be made -- if
     * any such manipulation is necessary, it should be done
     * on a copy of the array returned rather than the array 
     * itself.
     */
    public EventListener[] getListenerList() {
    	EventListener[] result = new EventListener[listenerList.length / 2];
    	int j=0;
    	for (int i = listenerList.length-2; i>=0;  i-=2) {
                result[j++] = (EventListener)listenerList[i+1];
        }
        return result;   
    }
    
    /**
     * Return an array of all the listeners of the given type. 
     * @return all of the listeners of the specified type. 
     * @exception  ClassCastException if the supplied class
     *		is not assignable to EventListener
     * 
     * @since 1.3
     */
    public EventListener[] getListeners(Event e) {
        Class t =e.getClass();
        return getListeners(t.getName());
    }
    
    public EventListener[] getListeners(Class e) {
        return getListeners(e.getName());
    }
    
    public EventListener[] getListeners(String e) {
        int n = getListenerCount(listenerList, e); 
        EventListener[] result = (EventListener[])Array.newInstance(EventListener.class, n); 
        int j = 0; 
        for (int i = listenerList.length-2; i>=0;  i-=2) {
            if (listenerList[i] == e) {
                result[j++] = (EventListener)listenerList[i+1];
            }
        }
        return result;   
    }
    
    /**
     * Returns the total number of listeners for this listener list.
     */
    public int getListenerCount() {
        return listenerList.length/2;
    }
    
    /**
     * Returns the total number of listeners of the supplied type 
     * for this listener list.
     */
    public int getListenerCount(Event e) {
        Class t = e.getClass();
        return getListenerCount(t.getName());
    }
    public int getListenerCount(Class e) {
        return getListenerCount(e.getName());           
    }
    
    public int getListenerCount(String e) {
        Object[] lList = listenerList;
        return getListenerCount(lList, e);
    }
    
    private int getListenerCount(Object[] list, String t) {
        int count = 0;
        for (int i = 0; i < list.length; i+=2) {
            if (t == list[i])
                count++;
        }
        return count;
    }
    
    /**
     * Adds the listener as a listener of the specified type.
     * @param t the type of the listener to be added
     * @param l the listener to be added
     */
    public synchronized void addListener(Event e, EventListener l) {
        if (!Event.class.isInstance(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event" );
        } else {
            Class t = e.getClass();
            addListener(t.getName(),l);
        }
    }
    
    public synchronized void addListener(Class e, EventListener l) {
        if (!Event.class.isAssignableFrom(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event class" );
        } else {
            addListener(e.getName(),l);
        }
    }
    
    public synchronized void addListener(String e, EventListener l) {
        if (logger.isDebugEnabled()) logger.info("Adding new Listener "+l+" of Event "+e);
        if (l==null) {
            // In an ideal world, we would do an assertion here
            // to help developers know they are probably doing
            // something wrong
            return;
        } 
        if (listenerList == NULL_ARRAY) {
            // if this is the first listener added, 
            // initialize the lists
            listenerList = new Object[] { e, l };
        } else {
            // Otherwise copy the array and add the new listener
            int i = listenerList.length;
            Object[] tmp = new Object[i+2];
            System.arraycopy(listenerList, 0, tmp, 0, i);
            tmp[i] = e;
            tmp[i+1] = l;
            listenerList = tmp;
        }
        if (logger.isDebugEnabled()) logger.info("Added "+this.toString());
    }
    
    /**
     * Removes the listener as a listener of the specified type.
     * @param t the type of the listener to be removed
     * @param l the listener to be removed
     */
    
    public synchronized void removeListener(Event e, EventListener l) {
        if (!Event.class.isInstance(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event" );
        } else {
            Class t = e.getClass();
            removeListener(t.getName(),l);
        }
    }
    
    public synchronized void removeListener(Class e, EventListener l) {
        if (!Event.class.isAssignableFrom(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event class" );
        } else {
            removeListener(e.getName(),l);
        }
    }
    
    public synchronized void removeListener(String e, EventListener l) {
        if (l ==null) {
            // In an ideal world, we would do an assertion here
            // to help developers know they are probably doing
            // something wrong
            return;
        }
        // Is l on the list?
        int index = -1;
        for (int i = listenerList.length-2; i>=0; i-=2) {
            if ((listenerList[i]==e) && (listenerList[i+1].equals(l) == true)) {
                index = i;
                break;
            }
        }
        
        // If so,  remove it
        if (index != -1) {
            Object[] tmp = new Object[listenerList.length-2];
            // Copy the list up to index
            System.arraycopy(listenerList, 0, tmp, 0, index);
            // Copy from two past the index, up to
            // the end of tmp (which is two elements
            // shorter than the old list)
            if (index < tmp.length)
                System.arraycopy(listenerList, index+2, tmp, index, 
                        tmp.length - index);
            // set the listener array to the new array or null
            listenerList = (tmp.length == 0) ? NULL_ARRAY : tmp;
        }
    }
    
    /**
     * Returns a string representation of the EventListenerList.
     */
    public String toString() {
        Object[] lList = listenerList;
        String s = "\nEventListenerManager" +": ";
        s += lList.length/2 + " listener(s): \n";
        s += "\tNum\tEvent Class\t\t\t\t\t\t\t\tListener\n";
        for (int i = 0 ; i <= lList.length-2 ; i+=2) {
            s += "\t"+i+"\t";
            s += lList[i] +"\t\t";
            s += lList[i+1]+"\n";
        }
        return s;
    }
    
    public void fireEvent(Event e) {
        EventListener[] lList = getListeners(e); // not a very efficient implementation !
        if(logger.isDebugEnabled()) logger.info("Must notify"+lList.length+" "+lList);
        for (int i = lList.length-1; i>=0; i-=1) {
            if(logger.isDebugEnabled()) logger.info("Firing event "+e+" to listener "+lList[i]);
            lList[i].eventProcess(e); 
        }
    }
}
